@@ -1,6 +1,6 @@
 package Silki::Formatter::HTMLToWiki;
 BEGIN {
-  $Silki::Formatter::HTMLToWiki::VERSION = '0.03';
+  $Silki::Formatter::HTMLToWiki::VERSION = '0.04';
 }
 
 use strict;
@@ -11,7 +11,7 @@ use HTML::TreeBuilder;
 use IO::Handle;
 use Markdent::Types qw( OutputStream );
 use Silki::Formatter::HTMLToWiki::Table;
-use Silki::Types qw( Maybe Str ArrayRef PosOrZeroInt );
+use Silki::Types qw( Maybe Str ArrayRef PosOrZeroInt Bool );
 use Silki::Util qw( string_is_empty );
 use URI;
 
@@ -32,6 +32,12 @@ has _stream => (
     init_arg => undef,
 );
 
+has _last_output_was_newline => (
+    is      => 'rw',
+    isa     => Bool,
+    default => 0,
+);
+
 has _indent_level => (
     traits  => ['Counter'],
     is      => 'rw',
@@ -44,9 +50,16 @@ has _indent_level => (
     init_arg => undef,
 );
 
-has _bullet => (
-    is  => 'rw',
-    isa => Maybe[Str],
+has _bullet_stack => (
+    traits  => ['Array'],
+    is      => 'bare',
+    isa     => ArrayRef [Str],
+    default => sub { [] },
+    handles => {
+        _push_bullet => 'push',
+        _pop_bullet  => 'pop',
+        _bullet      => [ get => -1 ],
+    },
 );
 
 has _current_href => (
@@ -88,7 +101,7 @@ sub _replace_stream {
     my $self   = shift;
     my $buffer = shift;
 
-    open my $fh, '>', $buffer;
+    open my $fh, '>:utf8', $buffer;
 
     my $old_stream = $self->_stream();
 
@@ -296,46 +309,52 @@ sub _start_ul {
     my $self = shift;
 
     $self->_print_to_stream("\n")
-        if defined $self->_bullet();
-    $self->_set_bullet('*');
+        if defined $self->_bullet() && ! $self->_last_output_was_newline();
+    $self->_push_bullet('*');
     $self->_inc_indent_level();
 }
 
 sub _end_ul {
     my $self = shift;
 
-    $self->_set_bullet(undef);
+    $self->_pop_bullet();
     $self->_dec_indent_level();
-    $self->_print_to_stream("\n");
+    $self->_print_to_stream("\n")
+        unless $self->_indent_level();
 }
 
 sub _start_ol {
     my $self = shift;
 
     $self->_print_to_stream("\n")
-        if defined $self->_bullet();
-    $self->_set_bullet('1.');
+        if defined $self->_bullet() && ! $self->_last_output_was_newline();
+    $self->_push_bullet('1.');
     $self->_inc_indent_level();
 }
 
 sub _end_ol {
     my $self = shift;
 
-    $self->_set_bullet(undef);
+    $self->_pop_bullet();
     $self->_dec_indent_level();
-    $self->_print_to_stream("\n");
+    $self->_print_to_stream("\n")
+        unless $self->_indent_level();
 }
 
 sub _start_li {
     my $self = shift;
 
+    die "Attempt to start a list item but we are not in a list"
+        unless defined $self->_bullet();
+
     $self->_print_to_stream( q{ } x ( 4 * ( $self->_indent_level() - 1 ) ) );
     $self->_print_to_stream( $self->_bullet() . q{ } );
 }
 
 sub _end_li {
     my $self = shift;
-    $self->_print_to_stream("\n");
+    $self->_print_to_stream("\n")
+        unless $self->_last_output_was_newline();
 }
 
 sub _start_blockquote {
@@ -361,11 +380,14 @@ sub _handle_table {
     $table->finalize();
 
     $self->_print_to_stream( $table->as_markdown() );
+    $self->_print_to_stream("\n");
 }
 
 sub _print_to_stream {
     my $self = shift;
 
+    $self->_set_last_output_was_newline( $_[0] eq "\n" ? 1 : 0 );
+
     $self->_stream()->print( $_[0] );
 }
 
@@ -382,7 +404,7 @@ Silki::Formatter::HTMLToWiki
 
 =head1 VERSION
 
-version 0.03
+version 0.04
 
 =head1 AUTHOR
 
