@@ -1,6 +1,6 @@
 package Silki::Schema::PageRevision;
 BEGIN {
-  $Silki::Schema::PageRevision::VERSION = '0.03';
+  $Silki::Schema::PageRevision::VERSION = '0.04';
 }
 
 use strict;
@@ -23,7 +23,6 @@ use Silki::Markdent::Handler::HTMLStream;
 use Silki::Schema;
 use Silki::Schema::Page;
 use Silki::Schema::PageLink;
-use Silki::Schema::PageFileLink;
 use Silki::Schema::PendingPageLink;
 use Silki::Types qw( Bool );
 use Storable qw( nfreeze thaw );
@@ -69,9 +68,13 @@ after update => sub {
     $self->_post_change();
 };
 
+our $SkipPostChangeHack;
+
 sub _post_change {
     my $self = shift;
 
+    return if $SkipPostChangeHack;
+
     my ( $existing, $pending, $files, $capture )
         = $self->_process_extracted_links();
 
@@ -90,13 +93,6 @@ sub _post_change {
             '=', $self->page_id()
         );
 
-    my $delete_files = Silki::Schema->SQLFactoryClass()->new_delete();
-    $delete_files->delete()->from( $Schema->table('PageFileLink') )
-        ->where(
-            $Schema->table('PageFileLink')->column('page_id'),
-            '=', $self->page_id()
-        );
-
     my $update_cached_content
         = Silki::Schema->SQLFactoryClass()->new_update();
     $update_cached_content->update( $Schema->table('Page') )
@@ -121,11 +117,6 @@ sub _post_change {
             {},
             $delete_pending->bind_params()
         );
-        $dbh->do(
-            $delete_files->sql($dbh),
-            {},
-            $delete_files->bind_params()
-        );
 
         my $sth = $dbh->prepare( $update_cached_content->sql($dbh) );
         my @bind = $update_cached_content->bind_params();
@@ -137,18 +128,17 @@ sub _post_change {
             if @{$existing};
         Silki::Schema::PendingPageLink->insert_many( @{$pending} )
             if @{$pending};
-        Silki::Schema::PageFileLink->insert_many( @{$files} )
-            if @{$files};
     };
 
     Silki::Schema->RunInTransaction($updates);
 }
 
 sub _process_extracted_links {
-    my $self   = shift;
+    my $self = shift;
 
     my $capture = Markdent::Handler::CaptureEvents->new();
     my $linkex  = Silki::Markdent::Handler::ExtractWikiLinks->new(
+        page => $self->page(),
         wiki => $self->page()->wiki(),
     );
     my $multi = Markdent::Handler::Multiplexer->new(
@@ -266,6 +256,7 @@ sub content_as_html {
 
     my $html = Silki::Markdent::Handler::HTMLStream->new(
         output => $fh,
+        page   => $page,
         wiki   => $page->wiki(),
         %p,
     );
@@ -305,7 +296,7 @@ Silki::Schema::PageRevision - Represents a page revision
 
 =head1 VERSION
 
-version 0.03
+version 0.04
 
 =head1 AUTHOR
 
